@@ -49,21 +49,8 @@ from icepool.population.func import (d, __getattr__, coin, one_hot,
                                      cartesian_product,
                                      from_cumulative_quantities, from_rv,
                                      min_outcome, max_outcome, align,
-                                     align_range, reduce, accumulate, apply)
-
-apply_sorted = icepool.population.func.apply_sorted
-"""As apply() but sorts the outcomes first.
-
-This is more efficient than `apply()` but still not particularly efficient.
-
-You can use `apply_sorted[index]()` to only keep certain sorted indexes.
-See `Pool.keep()` for documentation.
-
-Args:
-    func: A function that takes one argument per input `Die` and returns an
-        argument to `Die()`.
-    *args: `func` will be called with all sorted joint outcomes of these.
-"""
+                                     align_range, reduce, accumulate, apply,
+                                     outcome_function)
 
 from icepool.population.base import Population
 from icepool.population.die import implicit_convert_to_die, Die
@@ -77,6 +64,7 @@ from icepool.population.keep import lowest, highest, middle
 from icepool.generator.pool import Pool, standard_pool
 
 from icepool.generator.multiset_generator import MultisetGenerator, NextMultisetGenerator
+from icepool.generator.alignment import Alignment
 from icepool.evaluator.multiset_evaluator import MultisetEvaluator
 
 from icepool.population.deck import Deck
@@ -96,7 +84,8 @@ __all__ = [
     'Again', 'CountsKeysView', 'CountsValuesView', 'CountsItemsView',
     'from_cumulative_quantities', 'from_rv', 'align', 'align_range', 'lowest',
     'highest', 'middle', 'min_outcome', 'max_outcome', 'reduce', 'accumulate',
-    'apply', 'apply_sorted', 'Reroll', 'RerollType', 'Pool', 'standard_pool',
-    'MultisetGenerator', 'MultisetExpression', 'MultisetEvaluator', 'Order',
-    'Deck', 'Deal', 'multiset_function', 'typing'
+    'apply', 'outcome_function', 'Reroll', 'RerollType', 'Pool',
+    'standard_pool', 'MultisetGenerator', 'Alignment', 'MultisetExpression',
+    'MultisetEvaluator', 'Order', 'Deck', 'Deal', 'multiset_function', 'typing',
+    'evaluator'
 ]
