@@ -6,7 +6,7 @@ import icepool
 from icepool.evaluator.multiset_evaluator import MultisetEvaluator
 
 from icepool.typing import Outcome, Order
-from typing import Any
+from typing import Any, Collection, Literal, Sequence
 
 
 class HighestOutcomeAndCountEvaluator(MultisetEvaluator[Any, tuple[Any, int]]):
@@ -29,7 +29,7 @@ class HighestOutcomeAndCountEvaluator(MultisetEvaluator[Any, tuple[Any, int]]):
         else:
             return state
 
-    def order(self):
+    def order(self) -> Literal[Order.Any]:
         """Allows any order."""
         return Order.Any
 
@@ -40,32 +40,39 @@ class AllCountsEvaluator(MultisetEvaluator[Any, tuple[int, ...]]):
     In other words, this produces tuples of the sizes of all matching sets.
     """
 
-    def __init__(self, *, positive_only: bool = True):
+    def __init__(self, *, filter: int | None = 1) -> None:
         """
         Args:
-            positive_only: If `True` (default), any zero and negative values
-                in the result are removed.
+            filter: Any counts below this value will not be in the output.
+                For example, `filter=2` will only produce pairs and better.
+                If `None`, no filtering will be done.
         """
-        self._positive_only = positive_only
+        self._filter = filter
 
     def next_state(self, state, outcome, count):
         """Implementation."""
-        state = (state or ()) + (count,)
-        return tuple(sorted(state))
+        state = state or ()
+        if self._filter is None or count >= self._filter:
+            state = state + (count,)
+            return tuple(sorted(state))
+        else:
+            return state
 
-    def final_outcome(self, final_state):
+    def final_outcome(self, final_state) -> tuple:
         """Implementation."""
         if final_state is None:
             return ()
-        if self._positive_only:
-            return tuple(x for x in final_state if x > 0)
         else:
             return final_state
 
-    def order(self):
+    def order(self) -> Literal[Order.Any]:
         """Allows any order."""
         return Order.Any
 
+    def alignment(self, outcomes: Sequence) -> Collection:
+        """Always sees zeros."""
+        return outcomes
+
 
 class LargestCountEvaluator(MultisetEvaluator[Any, int]):
     """The largest count of any outcome."""
@@ -74,7 +81,7 @@ class LargestCountEvaluator(MultisetEvaluator[Any, int]):
         """Implementation."""
         return max(state or count, count)
 
-    def order(self):
+    def order(self) -> Literal[Order.Any]:
         """Allows any order."""
         return Order.Any
 
@@ -86,7 +93,7 @@ class LargestCountAndOutcomeEvaluator(MultisetEvaluator[Any, tuple[int, Any]]):
         """Implementation."""
         return max(state or (count, outcome), (count, outcome))
 
-    def order(self):
+    def order(self) -> Literal[Order.Any]:
         """Allows any order."""
         return Order.Any
 
@@ -103,13 +110,13 @@ class LargestStraightEvaluator(MultisetEvaluator[int, int]):
             run = 0
         return max(best_run, run), run
 
-    def final_outcome(self, final_state):
+    def final_outcome(self, final_state) -> int:
         """Implementation."""
         if final_state is None:
             return 0
         return final_state[0]
 
-    def order(self):
+    def order(self) -> Literal[Order.Ascending]:
         """Ascending order."""
         return Order.Ascending
 
@@ -129,11 +136,11 @@ class LargestStraightAndOutcomeEvaluator(MultisetEvaluator[int, tuple[int,
             run = 0
         return max(best_run_and_outcome, (run, outcome)), run
 
-    def final_outcome(self, final_state):
+    def final_outcome(self, final_state) -> tuple[int, int]:
         """Implementation."""
         return final_state[0]
 
-    def order(self):
+    def order(self) -> Literal[Order.Ascending]:
         """Ascending order."""
         return Order.Ascending
 
