@@ -1,30 +1,24 @@
 __docformat__ = 'google'
 
-from icepool.typing import Outcome
+from icepool.typing import Outcome, Qs_co, T
 
-from typing import TypeVar
+from typing import Any, Hashable
 import icepool
-from icepool.counts import CountsKeysView
-from icepool.generator.outcome_count_generator import NextOutcomeCountGenerator, OutcomeCountGenerator
+from icepool.collections import CountsKeysView
+from icepool.generator.multiset_generator import NextMultisetGenerator, MultisetGenerator
 from icepool.math import iter_hypergeom
 
 from functools import cached_property
 import math
 
-T_co = TypeVar('T_co', bound=Outcome, covariant=True)
-"""Type variable representing the outcome type."""
 
-Q = TypeVar('Q', bound=tuple[int, ...])
-"""Type variable representing the count types. In this future this may be replaced with a TypeVarTuple."""
+class Deal(MultisetGenerator[T, Qs_co]):
+    """Represents an sorted/unordered deal of cards from a `Deck`."""
 
-
-class Deal(OutcomeCountGenerator[T_co, Q]):
-    """EXPERIMENTAL: Represents an sorted/unordered deal of cards from a `Deck`. """
-
-    _deck: 'icepool.Deck[T_co]'
+    _deck: 'icepool.Deck[T]'
     _hand_sizes: tuple[int, ...]
 
-    def __init__(self, deck: 'icepool.Deck[T_co]', *hand_sizes: int):
+    def __init__(self, deck: 'icepool.Deck[T]', *hand_sizes: int) -> None:
         """Constructor.
 
         For algorithmic reasons, you must pre-commit to the number of cards to
@@ -38,7 +32,7 @@ class Deal(OutcomeCountGenerator[T_co, Q]):
         Args:
             deck: The `Deck` to deal from.
             *hand_sizes: How many cards to deal. If multiple `hand_sizes` are
-                provided, `OutcomeCountEvaluator.next_state` will recieve one count
+                provided, `MultisetEvaluator.next_state` will recieve one count
                 per hand in order. Try to keep the number of hands to a minimum
                 as this can be computationally intensive.
         """
@@ -52,14 +46,14 @@ class Deal(OutcomeCountGenerator[T_co, Q]):
             )
 
     @classmethod
-    def _new_raw(cls, deck: 'icepool.Deck[T_co]',
-                 hand_sizes: Q) -> 'Deal[T_co, Q]':
+    def _new_raw(cls, deck: 'icepool.Deck[T]',
+                 hand_sizes: tuple[int, ...]) -> 'Deal[T, Any]':
         self = super(Deal, cls).__new__(cls)
         self._deck = deck
         self._hand_sizes = hand_sizes
         return self
 
-    def deck(self) -> 'icepool.Deck[T_co]':
+    def deck(self) -> 'icepool.Deck[T]':
         """The `Deck` the cards are dealt from."""
         return self._deck
 
@@ -71,7 +65,7 @@ class Deal(OutcomeCountGenerator[T_co, Q]):
         """The total number of cards dealt."""
         return sum(self.hand_sizes())
 
-    def outcomes(self) -> CountsKeysView[T_co]:
+    def outcomes(self) -> CountsKeysView[T]:
         """The outcomes of the `Deck` in sorted order.
 
         These are also the `keys` of the `Deck` as a `Mapping`.
@@ -79,7 +73,7 @@ class Deal(OutcomeCountGenerator[T_co, Q]):
         """
         return self.deck().outcomes()
 
-    def counts_len(self) -> int:
+    def output_arity(self) -> int:
         return len(self._hand_sizes)
 
     def _is_resolvable(self) -> bool:
@@ -97,8 +91,8 @@ class Deal(OutcomeCountGenerator[T_co, Q]):
     def denominator(self) -> int:
         return self._denomiator
 
-    def _generate_common(self, popped_deck: 'icepool.Deck[T_co]',
-                         deck_count: int) -> NextOutcomeCountGenerator:
+    def _generate_common(self, popped_deck: 'icepool.Deck[T]',
+                         deck_count: int) -> NextMultisetGenerator:
         """Common implementation for _generate_min and _generate_max."""
         min_count = max(
             0, deck_count + self.total_cards_dealt() - self.deck().size())
@@ -114,7 +108,7 @@ class Deal(OutcomeCountGenerator[T_co, Q]):
                 weight = weight_total * weight_split
                 yield popped_deal, counts, weight
 
-    def _generate_min(self, min_outcome) -> NextOutcomeCountGenerator:
+    def _generate_min(self, min_outcome) -> NextMultisetGenerator:
         if not self.outcomes() or min_outcome != self.min_outcome():
             yield self, (0,), 1
             return
@@ -123,7 +117,7 @@ class Deal(OutcomeCountGenerator[T_co, Q]):
 
         yield from self._generate_common(popped_deck, deck_count)
 
-    def _generate_max(self, max_outcome) -> NextOutcomeCountGenerator:
+    def _generate_max(self, max_outcome) -> NextMultisetGenerator:
         if not self.outcomes() or max_outcome != self.max_outcome():
             yield self, (0,), 1
             return
@@ -137,21 +131,9 @@ class Deal(OutcomeCountGenerator[T_co, Q]):
         return result, result
 
     @cached_property
-    def _key_tuple(self) -> tuple:
+    def _key_tuple(self) -> tuple[Hashable, ...]:
         return Deal, self.deck(), self.hand_sizes()
 
-    def equals(self, other) -> bool:
-        if not isinstance(other, Deal):
-            return False
-        return self._key_tuple == other._key_tuple
-
-    @cached_property
-    def _hash(self) -> int:
-        return hash(self._key_tuple)
-
-    def __hash__(self) -> int:
-        return self._hash
-
     def __repr__(self) -> str:
         return type(
             self
